/* @flow */
import type ModuleBase from '../utils/ModuleBase';

/* Core types */
export type FirebaseError = {
  message: string,
  name: string,
  code: string,
  stack: string,
  path: string,
  details: string,
  modifiers: string
}

export type FirebaseModule = $Subtype<ModuleBase>;

export type FirebaseOptions = {
  apiKey: string,
  appId: string,
  databaseURL: string,
  messagingSenderId: string,
  projectId: string,
  storageBucket: string,
}

export type FirebaseModuleAndStatics<M: FirebaseModule, S> = {
  (): M,
  nativeModuleExists: boolean,
} & S;

/* Auth types */

export type AuthCredential = {
  providerId: string,
  token: string,
  secret: string
}

/* Database types */

export type DatabaseModifier = {
  type: 'orderBy' | 'limit' | 'filter';
  name?: string;
  key?: string;
  limit?: number;
  value?: any;
  valueType?: string;
}

/* Firestore types */

export type FirestoreNativeDocumentChange = {
  document: FirestoreNativeDocumentSnapshot,
  newIndex: number,
  oldIndex: number,
  type: string,
}

export type FirestoreNativeDocumentSnapshot = {
  data: { [string]: FirestoreTypeMap },
  metadata: FirestoreSnapshotMetadata,
  path: string,
}

export type FirestoreSnapshotMetadata = {
  fromCache: boolean,
  hasPendingWrites: boolean,
}

export type FirestoreQueryDirection = 'DESC' | 'desc' | 'ASC' | 'asc';
export type FirestoreQueryOperator = '<' | '<=' | '=' | '==' | '>' | '>=';

export type FirestoreTypeMap = {
  type: 'array' | 'boolean' | 'date' | 'fieldvalue' | 'geopoint' | 'null' | 'number' | 'object' | 'reference' | 'string',
  value: any,
}

export type FirestoreWriteOptions = {
  merge?: boolean,
}
